;*=*=*
;       EMOUSE1/CMD
;       LS-DOS driver for TRS80 emulation of mouse (David Goben compatable)
;
;       Copyright (c) 1998, Timothy Mann
;
;       This software may be copied, modified, and used for any
;       purpose without fee, provided that (1) the above copyright
;       notice is retained, and (2) modified versions are clearly
;       marked as having been modified, with the modifier's name and
;       the date included.  
;
;       Created 9-28-98
;       modified on Sat Oct 10 20:57:36 PDT 1998 by mann
;       Last modified on 05/14/99 by D. Keil
;       (changed name and hello message also made compatible with RASM)
;
;       Usage:
;         emouse1              To load driver in high memory.
;         emouse1 (low)        To load driver in low memory if possible,
;                                 or in high memory if low memory is full.
;
;       The default is to use high memory because MDRAW/BAS contains a
;       PEEK in the first line of code that looks for the driver in
;       high memory; if it is in low memory, MDRAW thinks the driver
;       is not installed and exits.  If you edit this line of code to
;       remove the test, the driver will work fine in low memory.
;*=*=*
; ASCII chars
LF      EQU     10
CR      EQU     13
ETX     EQU     3
; Model 4 SVC numbers
@HIGH   EQU     100
@DSPLY  EQU     10
@FLAGS  EQU     101
@LOGOT  EQU     12
@GTDCB  EQU     82 
@GTMOD  EQU     83
@KEYIN  EQU     9
@PARAM  EQU     17
@MOUSE  EQU     120     ;unofficial value
SVCVEC  EQU     2*@MOUSE+100H
; xtrs emts (byte-reversed)
EMT_MOUSE  EQU  029EDH
        ORG     3000H
;*=*=*
;       Relocator for disk driver
;*=*=*
INSTAL: LD      DE,PRMTAB
        LD      A,@PARAM        ;Parse parameters
        RST     40
        JP      NZ,PRMERR
        LD      HL,HELLO_
        LD      A,@DSPLY        ;Display hello
        RST     40
;*=*=*
;       Check if driver already loaded
;*=*=*
        LD      DE,MODNAM
        LD      A,@GTMOD
        RST     40
        JP      Z,LOADED        ;Already loaded
;*=*=*
;       Check if OK to use low memory.
;*=*=*
LPARM:  LD      BC,0
        LD      A,B
        OR      C
        JR      Z,USEHI
;*=*=*
;       Obtain low memory driver pointer.  Bizarre API here!
;*=*=*
        LD      E,'K'           ;Locate pointer to *KI DCB
        LD      D,'I'           ;  via @GTDCB SVC
        LD      A,@GTDCB        
        RST     40
        JP      NZ,CURDL        ;No error unless KI clobbered!
        DEC     HL              ;Decrement to driver pointer
        LD      D,(HL)          ;P/u hi-order of pointer,
        DEC     HL              ;  decrement to and p/u
        LD      E,(HL)          ;  lo-order of pointer
;*=*=*
;       Check if driver will fit into [(LCPTR), X'12FF']
;*=*=*
        PUSH    HL              ;Save address of pointer
        LD      HL,LENGTH       ;New pointer will be
        ADD     HL,DE           ;  pointer + LENGTH
        LD      D,H             ;Save a copy in DE
        LD      E,L
        LD      BC,1301H        ;If > 1300H, driver won't fit
        SUB     A               ;Reset carry flag
        SBC     HL,BC
        POP     HL              ;Get back address of pointer
        JR      NC,USEHI        ;Go if driver won't fit
        LD      (HL),E          ;Store new value of pointer
        INC     HL
        LD      (HL),D
        DEC     DE              ;Last byte of driver goes here
        LD      (NEWEND),DE
        JR      DORELO
;*=*=*
;       Put in high memory instead.
;*=*=*
USEHI:  LD      HL,0            ;Get current HIGH$
        LD      B,L
        LD      A,@HIGH 
        RST     40
        JP      NZ,NOMEM
        LD      (NEWEND),HL     ;Last byte of driver goes here
        LD      DE,LENGTH
        SUB     A               ;Reset carry flag
        SBC     HL,DE           ;Compute new HIGH$
        LD      A,@HIGH         ;Set new HIGH$ into the system
        RST     40
;*=*=*
;       Relocate internal references in driver.
;       HL = address for last byte of driver.
;*=*=*
DORELO: CALL    RELO
;*=*=*
;       Link to @ICNFG (must follow address relocation and precede movement)
;*=*=*
        LD      A,@FLAGS        ;Get flags pointer into IY
        RST     40
        LD      A,(IY+28)       ;Copy current @ICNFG into LINK
        LD      L,(IY+29)
        LD      H,(IY+30)
        LD      (LINK),A
        LD      (LINK+1),HL
        LD      HL,DVRCFG       ;Get relocated init address
RX01    EQU     $-2
        LD      (IY+29),L       ;Save in @ICNFG vector
        LD      (IY+30),H
        LD      (IY+28),0C3H    ;Insert JP opcode
;*=*=*
;       Move driver into low or high memory.
;*=*=*
MOVE:
        LD      DE,(NEWEND)     ;Destination address
        LD      HL,DVREND       ;Last byte of module
        LD      BC,LENGTH       ;Length of filter
        LDDR
        EX      DE,HL
        INC     HL              ;Bump to driver entry
;*=*=*
;       Driver is loaded; finish up.
;*=*=*
        CALL    DVRINI          ;Hook into SVC table
        LD      HL,0            ;Successful completion
        SUB     A
        RET
;*=*=*
;       Error vectors
;*=*=*
PRMERR: LD      HL,PRMERR_
        DEFB    0DDH
LOADED: LD      HL,LOADED_
        DEFB    0DDH
CURDL:  LD      HL,CURDL_
        DEFB    0DDH
NOMEM:  LD      HL,NOMEM_
LOGOT:  LD      A,@LOGOT 
        RST     40
        LD      HL,-1            ;Unuccessful completion
        RET
;*=*=*
;       Relocate internal references in driver.
;       HL = address for last byte of driver.
;*=*=*
RELO:   LD      HL,(NEWEND)
        LD      IY,RELTAB       ;Point to relocation tbl
        LD      DE,DVREND
        SUB     A               ;Clear carry flag
        SBC     HL,DE
        LD      B,H             ;Move to BC
        LD      C,L
RLOOP:  LD      L,(IY)          ;Get address to change
        LD      H,(IY+1)
        LD      A,H
        OR      L
        RET     Z
        LD      E,(HL)          ;P/U address
        INC     HL
        LD      D,(HL)
        EX      DE,HL           ;Offset it
        ADD     HL,BC
        EX      DE,HL
        LD      (HL),D          ;And put back
        DEC     HL
        LD      (HL),E
        INC     IY
        INC     IY
        JR      RLOOP           ;Loop till done
;*=*=*
;       Messages and globals
;*=*=*
HELLO_: DEFB    'EMOUSE1 - A "David Goben" compatable mouse driver for TRS80 - 5/14/99',LF
        DEFB    'SVC number 120 has been installed.',CR
CURDL_: DEFB    'LS-DOS is curdled!',CR
NOMEM_: DEFB    'High memory is not available!',CR
LOADED_:DEFB    'Mouse driver is already loaded!',CR
PRMERR_:DEFB    'Bad parameters!',CR
LCPTR:  DEFW    0
NEWEND: DEFW    0
;*=*=*
;       Parameter table
;*=*=*
PRMTAB: DEFB    'LOW   '
        DEFW    LPARM+1
        DEFB    'L     '
        DEFW    LPARM+1
        DEFB    0
;*=*=*
;       Driver
;*=*=*
ENTRY:  JR      BEGIN           ;The driver starts with the
        DEFW    DVREND          ;  DOS standard header
RX00    EQU     $-2
        DEFB    MODPTR-MODNAM   ;Length of name
MODNAM: DEFB    '$MOUSE'        ;Name for @GTMOD requests
MODPTR: DEFW    0               ;These pointers are unused, but 1st byte MBZ
        DEFW    0
;*=*=*
;       Do the real work using an emulator trap
;*=*=*
BEGIN:  DEFW    EMT_MOUSE
        RET
;*=*=*
;       Boot-time initialization
;*=*=*
DVRCFG:                         ;@ICNFG chains in here
        CALL    DVRINI
RX02    EQU     $-2     
LINK:   DEFB    'Tim'           ;Replaced by next link in @ICNFG chain
;*=*=*
;       Hook into SVC table
;*=*=*
DVRINI: LD      HL,ENTRY
RX03    EQU     $-2
        LD      (SVCVEC),HL
        RET
DVREND  EQU     $-1
LENGTH  EQU     $-ENTRY
RELTAB: DEFW    RX00,RX01,RX02,RX03,0
        END     INSTAL
